==> simple_http_client-0.9.0.tar.gz: test/simple_client_test.dart <==
/* Copyright (c) 2017 Miguel Castiblanco */
import 'dart:async';

import 'package:resource/resource.dart' show Resource;
import 'package:test/test.dart';
import 'package:simple_http_client/simple_http_client.dart';
import 'package:mock_web_server/mock_web_server.dart';

import 'dart:io';

MockWebServer _server;
bool mocked = true;

main() {
  setUp(() async {
    _server = new MockWebServer();
    await _server.start();
  });

  tearDown(() {
    _server.shutdown();
  });

  test('simple request', () async {
    String responseBody = "Hello World";
    _server.enqueue(body: responseBody);

    SimpleHttpClient client = new SimpleHttpClient();

    String response = (await client.get(_server.url)).body;
    expect(responseBody, response);
    expect(_server.requestCount, 1);
  });

  test('methods', () async {
    _server.enqueue();
    _server.enqueue();
    _server.enqueue();
    _server.enqueue();

    SimpleHttpClient client = new SimpleHttpClient();

    await client.get(_server.url);
    expect(_server.takeRequest().method, "GET");

    await client.post(_server.url, "");
    expect(_server.takeRequest().method, "POST");

    await client.put(_server.url, "");
    expect(_server.takeRequest().method, "PUT");

    await client.delete(_server.url);
    expect(_server.takeRequest().method, "DELETE");

    expect(_server.requestCount, 4);
  });

  test('status code read', () async {
    _server.enqueue(httpCode: HttpStatus.noContent);
    _server.enqueue(httpCode: HttpStatus.accepted);
    _server.enqueue(httpCode: HttpStatus.badRequest);

    SimpleHttpClient client = new SimpleHttpClient();

    var res = await client.get(_server.url);
    expect(res.statusCode, HttpStatus.noContent);

    res = await client.post(_server.url, "");
    expect(res.statusCode, HttpStatus.accepted);

    res = await client.put(_server.url, "");
    expect(res.statusCode, HttpStatus.badRequest);

    expect(_server.requestCount, 3);
  });

  test('simple intercept', () async {
    String responseBody = "Intercepted Response";
    String headerAuth = "BEARER XAZSA";

    _server.enqueue(body: headerAuth, delay: new Duration(milliseconds: 200));
    _server.enqueue(body: responseBody);

    SimpleHttpClient client = new SimpleHttpClient();
    client.addInterceptor((HttpClientRequest request) async {
      // Request and add the auth header only to the first request
      if ("/user/1" == request.uri.path) {
        String token = (await client.get(_server.url)).body;
        request.headers.set("Authorization", token);
      }
    });

    String response = (await client.get(_server.url + "user/1")).body;
    expect(responseBody, response);
    expect(2, _server.requestCount);

    // discard second response
    _server.takeRequest();

    StoredRequest request = _server.takeRequest();
    expect(headerAuth, request.headers["authorization"]);
  });

  test('useragent', () async {
    String responseBody = "Response";
    String userAgent = "Awesome client";
    _server.enqueue(body: responseBody);

    SimpleHttpClient client = new SimpleHttpClient(userAgent: userAgent);

    String response = (await client.get(_server.url)).body;
    expect(responseBody, response);
    expect(1, _server.requestCount);

    StoredRequest request = _server.takeRequest();
    expect(userAgent, request.headers["user-agent"]);
  });

  test('authenticator', () async {
    String body = "Welcome!";
    String header = "BEARER JAHDHSA";
    String requestBody = "Retried";
    _server.enqueue(httpCode: HttpStatus.unauthorized);
    _server.enqueue(body: body);

    SimpleHttpClient client = new SimpleHttpClient();
    client.setAuthenticator((request) {
      request.headers.set("Authorization", header);
    });

    String response = (await client.post(_server.url, requestBody)).body;
    expect(body, response);
    expect(_server.requestCount, 2);

    StoredRequest unauthorized = _server.takeRequest();
    StoredRequest success = _server.takeRequest();

    //expect(HttpStatus.UNAUTHORIZED, unauthorized.response.statusCode);
    expect(unauthorized.headers["authorization"], null);

    //expect(HttpStatus.OK, success.response.statusCode);
    expect(header, success.headers["authorization"]);
  });

  test('max auth retries', () async {
    _server.enqueue(httpCode: HttpStatus.unauthorized);
    _server.enqueue(httpCode: HttpStatus.unauthorized);
    _server.enqueue(httpCode: HttpStatus.unauthorized);

    SimpleHttpClient client = new SimpleHttpClient(maxAuthRetries: 2);
    client.setAuthenticator((request) {
      // Do nothing
    });

    await expectLater(client.get(_server.url),
        throwsA(new TypeMatcher<UnauthorizedException>()));

    expect(_server.requestCount, 3);
  });

  test('UnauthorizedException', () async {
    _server.enqueue(httpCode: HttpStatus.unauthorized);

    SimpleHttpClient client = new SimpleHttpClient();

    expect(client.get(_server.url),
        throwsA(new TypeMatcher<UnauthorizedException>()));
  });

  test('CantConnectException', () async {
    SimpleHttpClient client = new SimpleHttpClient();

    expect(client.get("http://l.o.c.alhost:6453/"),
        throwsA(new TypeMatcher<CantConnectException>()));
  });

  test('timeout', () async {
    String body = "51 milliseconds";
    _server.enqueue(delay: new Duration(milliseconds: 70), body: body);

    SimpleHttpClient client =
        new SimpleHttpClient(timeout: new Duration(milliseconds: 50));

    expect(
        client.get(_server.url), throwsA(new TypeMatcher<TimeOutException>()));
  });

  test('headers send', () async {
    String name = "Awesome header";
    _server.enqueue();
    _server.enqueue();
    _server.enqueue();
    _server.enqueue();

    SimpleHttpClient client = new SimpleHttpClient();

    await client.get(_server.url, headers: {"name": name});
    expect(_server.takeRequest().headers["name"], name);
  });

  test('headers retrieve', () async {
    _server.enqueue(headers: {"x-my-header": "awesome"});
    _server.enqueue(headers: {"x-my-header": "awesome, double"});

    SimpleHttpClient client = new SimpleHttpClient();

    var res = await client.get(_server.url);
    expect(res.headers["x-my-header"], "awesome");

    res = await client.get(_server.url);
    expect(res.headers["x-my-header"], "awesome, double");
  });

  test('headers retry', () async {
    String name = "Awesome header";
    _server.enqueue(httpCode: HttpStatus.unauthorized);
    _server.enqueue();

    SimpleHttpClient client = new SimpleHttpClient();
    client.setAuthenticator((request) {
      // Do nothing
    });

    await client.get(_server.url, headers: {"name": name});
    expect(_server.requestCount, 2);
    expect(_server.takeRequest().headers["name"], name);
    expect(_server.takeRequest().headers["name"], name);
  });

  test('parallel reqs', () async {
    String body70 = "70 milliseconds";
    String body40 = "40 milliseconds";
    String body20 = "20 milliseconds";
    _server.enqueue(delay: new Duration(milliseconds: 40), body: body40);
    _server.enqueue(delay: new Duration(milliseconds: 70), body: body70);
    _server.enqueue(delay: new Duration(milliseconds: 20), body: body20);

    Completer completer = new Completer();
    List<String> responses = new List();

    SimpleHttpClient client = new SimpleHttpClient();

    client.get(_server.url).then((res) async {
      // 40 milliseconds
      responses.add(res.body);
    });

    client.get(_server.url).then((res) async {
      // 70 milliseconds
      responses.add(res.body);

      // complete on the longer operation
      completer.complete();
    });

    client.get(_server.url).then((res) async {
      // 20 milliseconds
      responses.add(res.body);
    });

    await completer.future;

    // validate that the responses happened in order 20, 40, 70
    expect(responses[0], body20);
    expect(responses[1], body40);
    expect(responses[2], body70);
  });

  test('TLS any cert', () async {
    var body = "S03E08 You Are Not Safe";

    var chainRes =
        new Resource('package:mock_web_server/certificates/server_chain.pem');
    var chain = chainRes.readAsBytes();

    var keyRes =
        new Resource('package:mock_web_server/certificates/server_key.pem');
    var key = keyRes.readAsBytes();

    Certificate certificate = new Certificate()
      ..password = "dartdart"
      ..key = await key
      ..chain = await chain;

    var _server = new MockWebServer(certificate: certificate);

    await _server.start();
    _server.enqueue(body: body);

    var client = new SimpleHttpClient(allowAnyCert: true);
    var res = await client.get(_server.url);

    expect(res.body, body);
  });

  test('TLS validate cert and fail', () async {
    var body = "S03E08 You Are Not Safe";

    var chainRes =
        new Resource('package:mock_web_server/certificates/server_chain.pem');
    var chain = chainRes.readAsBytes();

    var keyRes =
        new Resource('package:mock_web_server/certificates/server_key.pem');
    var key = keyRes.readAsBytes();

    Certificate certificate = new Certificate()
      ..password = "dartdart"
      ..key = await key
      ..chain = await chain;

    var _server = new MockWebServer(certificate: certificate);

    await _server.start();
    _server.enqueue(body: body);

    var client = new SimpleHttpClient();

    expect(client.get(_server.url), throwsA(new TypeMatcher<TLSException>()));
  });

  test('TLS custom cert validation', () async {
    var body = "S03E08 You Are Not Safe";

    var chainRes =
        new Resource('package:mock_web_server/certificates/server_chain.pem');
    var chain = chainRes.readAsBytes();

    var keyRes =
        new Resource('package:mock_web_server/certificates/server_key.pem');
    var key = keyRes.readAsBytes();

    Certificate certificate = new Certificate()
      ..password = "dartdart"
      ..key = await key
      ..chain = await chain;

    var _server = new MockWebServer(certificate: certificate);

    await _server.start();
    _server.enqueue(body: body);

    var certRes =
        new Resource('package:mock_web_server/certificates/trusted_certs.pem');
    List<int> cert = await certRes.readAsBytes();

    var client = new SimpleHttpClient(
        trustedCertificates: [new SimpleCertificate(cert)]);
    var res = await client.get(_server.url);

    expect(res.body, body);
  });
}

==> simple_http_client-0.9.9.tar.gz: test/simple_client_test.dart <==
/* Copyright (c) 2017 Miguel Castiblanco */
import 'dart:async';

import 'package:resource/resource.dart' show Resource;
import 'package:test/test.dart';
import 'package:simple_http_client/simple_http_client.dart';
import 'package:mock_web_server/mock_web_server.dart';

import 'dart:io';

MockWebServer _server;
bool mocked = true;

main() {
  setUp(() async {
    _server = new MockWebServer();
    await _server.start();
  });

  tearDown(() {
    _server.shutdown();
  });

  test('simple request', () async {
    String responseBody = "Hello World";
    _server.enqueue(body: responseBody);

    SimpleHttpClient client = new SimpleHttpClient();

    String response = (await client.get(_server.url)).body;
    expect(responseBody, response);
    expect(_server.requestCount, 1);
  });

  test('methods', () async {
    _server.enqueue();
    _server.enqueue();
    _server.enqueue();
    _server.enqueue();

    SimpleHttpClient client = new SimpleHttpClient();

    await client.get(_server.url);
    expect(_server.takeRequest().method, "GET");

    await client.post(_server.url, "");
    expect(_server.takeRequest().method, "POST");

    await client.put(_server.url, "");
    expect(_server.takeRequest().method, "PUT");

    await client.delete(_server.url);
    expect(_server.takeRequest().method, "DELETE");

    expect(_server.requestCount, 4);
  });

  test('status code read', () async {
    _server.enqueue(httpCode: HttpStatus.noContent);
    _server.enqueue(httpCode: HttpStatus.accepted);
    _server.enqueue(httpCode: HttpStatus.badRequest);

    SimpleHttpClient client = new SimpleHttpClient();

    var res = await client.get(_server.url);
    expect(res.statusCode, HttpStatus.noContent);

    res = await client.post(_server.url, "");
    expect(res.statusCode, HttpStatus.accepted);

    res = await client.put(_server.url, "");
    expect(res.statusCode, HttpStatus.badRequest);

    expect(_server.requestCount, 3);
  });

  test('simple request intercept', () async {
    String responseBody = "Intercepted Response";
    String headerAuth = "BEARER XAZSA";

    _server.enqueue(body: headerAuth, delay: new Duration(milliseconds: 200));
    _server.enqueue(body: responseBody);

    SimpleHttpClient client = new SimpleHttpClient();
    client.addRequestInterceptor((HttpClientRequest request) async {
      // Request and add the auth header only to the first request
      if ("/user/1" == request.uri.path) {
        String token = (await client.get(_server.url)).body;
        request.headers.set("Authorization", token);
      }

      return request;
    });

    String response = (await client.get(_server.url + "user/1")).body;
    expect(responseBody, response);
    expect(2, _server.requestCount);

    // discard second response
    _server.takeRequest();

    StoredRequest request = _server.takeRequest();
    expect(headerAuth, request.headers["authorization"]);
  });

  test('simple response intercept', () async {
    String responseBody = "Intercepted Response";
    _server.enqueue(body: responseBody);

    SimpleHttpClient client = new SimpleHttpClient();
    Completer completer = Completer<bool>();
    client.addResponseInterceptor((HttpClientRequest request, Response response) async {
      expect(response.body, responseBody);
      expect(response.statusCode, 200);

      expect(request.uri.toString(), _server.url + "user/1");

      completer.complete(true);
    });

    await client.get(_server.url + "user/1");
    await completer.future;
  });

  test('useragent', () async {
    String responseBody = "Response";
    String userAgent = "Awesome client";
    _server.enqueue(body: responseBody);

    SimpleHttpClient client = new SimpleHttpClient(userAgent: userAgent);

    String response = (await client.get(_server.url)).body;
    expect(responseBody, response);
    expect(1, _server.requestCount);

    StoredRequest request = _server.takeRequest();
    expect(userAgent, request.headers["user-agent"]);
  });

  test('authenticator', () async {
    String body = "Welcome!";
    String header = "BEARER JAHDHSA";
    String requestBody = "Retried";
    _server.enqueue(httpCode: HttpStatus.unauthorized);
    _server.enqueue(body: body);

    SimpleHttpClient client = new SimpleHttpClient();
    client.setAuthenticator((request) async {
      request.headers.set("Authorization", header);

      return request;
    });

    String response = (await client.post(_server.url, requestBody)).body;
    expect(body, response);
    expect(_server.requestCount, 2);

    StoredRequest unauthorized = _server.takeRequest();
    StoredRequest success = _server.takeRequest();

    //expect(HttpStatus.UNAUTHORIZED, unauthorized.response.statusCode);
    expect(unauthorized.headers["authorization"], null);

    //expect(HttpStatus.OK, success.response.statusCode);
    expect(header, success.headers["authorization"]);
  });

  test('max auth retries', () async {
    _server.enqueue(httpCode: HttpStatus.unauthorized);
    _server.enqueue(httpCode: HttpStatus.unauthorized);
    _server.enqueue(httpCode: HttpStatus.unauthorized);

    SimpleHttpClient client = new SimpleHttpClient(maxAuthRetries: 2);
    client.setAuthenticator((request) async {
      // Do nothing
      return request;
    });

    await expectLater(client.get(_server.url),
        throwsA(new TypeMatcher<UnauthorizedException>()));

    expect(_server.requestCount, 3);
  });

  test('UnauthorizedException', () async {
    _server.enqueue(httpCode: HttpStatus.unauthorized);

    SimpleHttpClient client = new SimpleHttpClient();

    expect(client.get(_server.url),
        throwsA(new TypeMatcher<UnauthorizedException>()));
  });

  test('CantConnectException', () async {
    SimpleHttpClient client = new SimpleHttpClient();

    expect(client.get("http://l.o.c.alhost:6453/"),
        throwsA(new TypeMatcher<CantConnectException>()));
  });

  test('timeout', () async {
    String body = "51 milliseconds";
    _server.enqueue(delay: new Duration(milliseconds: 70), body: body);

    SimpleHttpClient client =
        new SimpleHttpClient(timeout: new Duration(milliseconds: 50));

    expect(
        client.get(_server.url), throwsA(new TypeMatcher<TimeOutException>()));
  });

  test('headers send', () async {
    String name = "Awesome header";
    _server.enqueue();
    _server.enqueue();
    _server.enqueue();
    _server.enqueue();

    SimpleHttpClient client = new SimpleHttpClient();

    await client.get(_server.url, headers: {"name": name});
    expect(_server.takeRequest().headers["name"], name);
  });

  test('headers retrieve', () async {
    _server.enqueue(headers: {"x-my-header": "awesome"});
    _server.enqueue(headers: {"x-my-header": "awesome, double"});

    SimpleHttpClient client = new SimpleHttpClient();

    var res = await client.get(_server.url);
    expect(res.headers["x-my-header"], "awesome");

    res = await client.get(_server.url);
    expect(res.headers["x-my-header"], "awesome, double");
  });

  test('headers retry', () async {
    String name = "Awesome header";
    _server.enqueue(httpCode: HttpStatus.unauthorized);
    _server.enqueue();

    SimpleHttpClient client = new SimpleHttpClient();
    client.setAuthenticator((request) async {
      // Do nothing
      return request;
    });

    await client.get(_server.url, headers: {"name": name});
    expect(_server.requestCount, 2);
    expect(_server.takeRequest().headers["name"], name);
    expect(_server.takeRequest().headers["name"], name);
  });

  test('parallel reqs', () async {
    String body70 = "70 milliseconds";
    String body40 = "40 milliseconds";
    String body20 = "20 milliseconds";
    _server.enqueue(delay: new Duration(milliseconds: 40), body: body40);
    _server.enqueue(delay: new Duration(milliseconds: 70), body: body70);
    _server.enqueue(delay: new Duration(milliseconds: 20), body: body20);

    Completer completer = new Completer();
    List<String> responses = new List();

    SimpleHttpClient client = new SimpleHttpClient();

    client.get(_server.url).then((res) async {
      // 40 milliseconds
      responses.add(res.body);
    });

    client.get(_server.url).then((res) async {
      // 70 milliseconds
      responses.add(res.body);

      // complete on the longer operation
      completer.complete();
    });

    client.get(_server.url).then((res) async {
      // 20 milliseconds
      responses.add(res.body);
    });

    await completer.future;

    // validate that the responses happened in order 20, 40, 70
    expect(responses[0], body20);
    expect(responses[1], body40);
    expect(responses[2], body70);
  });

  test('TLS any cert', () async {
    var body = "S03E08 You Are Not Safe";

    var chainRes =
        new Resource('package:mock_web_server/certificates/server_chain.pem');
    var chain = chainRes.readAsBytes();

    var keyRes =
        new Resource('package:mock_web_server/certificates/server_key.pem');
    var key = keyRes.readAsBytes();

    Certificate certificate = new Certificate()
      ..password = "dartdart"
      ..key = await key
      ..chain = await chain;

    var _server = new MockWebServer(certificate: certificate);

    await _server.start();
    _server.enqueue(body: body);

    var client = new SimpleHttpClient(allowAnyCert: true);
    var res = await client.get(_server.url);

    expect(res.body, body);
  });

  test('TLS validate cert and fail', () async {
    var body = "S03E08 You Are Not Safe";

    var chainRes =
        new Resource('package:mock_web_server/certificates/server_chain.pem');
    var chain = chainRes.readAsBytes();

    var keyRes =
        new Resource('package:mock_web_server/certificates/server_key.pem');
    var key = keyRes.readAsBytes();

    Certificate certificate = new Certificate()
      ..password = "dartdart"
      ..key = await key
      ..chain = await chain;

    var _server = new MockWebServer(certificate: certificate);

    await _server.start();
    _server.enqueue(body: body);

    var client = new SimpleHttpClient();

    expect(client.get(_server.url), throwsA(new TypeMatcher<TLSException>()));
  });

  test('TLS custom cert validation', () async {
    var body = "S03E08 You Are Not Safe";

    var chainRes =
        new Resource('package:mock_web_server/certificates/server_chain.pem');
    var chain = chainRes.readAsBytes();

    var keyRes =
        new Resource('package:mock_web_server/certificates/server_key.pem');
    var key = keyRes.readAsBytes();

    Certificate certificate = new Certificate()
      ..password = "dartdart"
      ..key = await key
      ..chain = await chain;

    var _server = new MockWebServer(certificate: certificate);

    await _server.start();
    _server.enqueue(body: body);

    var certRes =
        new Resource('package:mock_web_server/certificates/trusted_certs.pem');
    List<int> cert = await certRes.readAsBytes();

    var client = new SimpleHttpClient(
        trustedCertificates: [new SimpleCertificate(cert)]);
    var res = await client.get(_server.url);

    expect(res.body, body);
  });
}
